//go build -ldflags "-s -w" server.go
package main

import (
	"fmt"
	"os"
	"errors"
	"time"
	"io"
	"io/ioutil"
	"path/filepath"
	"sync"
	"strconv"
	"strings"
	"net"
	"net/http"
	"net/url"
	"encoding/json"
)

var (
	Hosts = map[string]bool {
		"api2.musical.ly": false,
		"api.tiktokv.com": false,
		"api-t2.tiktokv.com": false,
		"api-tt.hypstar.com": false,
		"api-t.tiktok.com": false,
		"api-m.tiktok.com": false,
		"m.tiktok.com": false,
		"www.tiktok.com": false,
	}
	Mutex sync.Mutex
)

func EnsureDirectory(directory string) error {
	var (
		s os.FileInfo
		e error
	)
	s, e = os.Stat(directory)
	if e != nil {
		if os.IsNotExist(e) {
			e = os.Mkdir(directory, 0700)
			if e != nil {
				return e
			}
		} else {
			return e
		}
	} else if !s.IsDir() {
		return errors.New("Cannot create a directory if a file with the same name exists.")
	}
	return nil
}

type AwemeData struct {
	Id string "json:\"id\""
	Uri string "json:\"uri\""
}

func DownloadAweme(filename string, uri string, referer string, cookie string) (bool, error) {
	var (
		e error
		r int
		f *os.File
		rs *http.Response
		rq *http.Request
	)
	_, e = os.Stat(filename)
	if e == nil {
		return true, nil
	} else if !os.IsNotExist(e) {
		return true, e
	}
	rq, e = http.NewRequest("GET", uri, nil)
	if e != nil {
		return false, e
	}
	rq.Header.Set("User-Agent", "okhttp")
	if len(referer) > 0 {
		rq.Header.Set("Referer", referer)
	}
	if len(cookie) > 0 {
		rq.Header.Set("Cookie", cookie)
	}
	for ; r < 3; r++ {
		rs, e = http.DefaultClient.Do(rq)
		if e != nil {
			return false, e
		} else if rs.StatusCode == http.StatusOK && rs.ContentLength > 0 {
			defer rs.Body.Close()
			f, e = os.Create(filename)
			if e != nil {
				return false, e
			}
			_, e = io.Copy(f, rs.Body)
			f.Close()
			if e != nil {
				_ = os.Remove(filename)
				return false, e
			}
			return false, nil
		}
		rs.Body.Close()
	}
	if rs.StatusCode != http.StatusOK {
		return false, errors.New(rs.Status)
	} else {
		return false, errors.New("Response body is empty.")
	}
}

func DownloadVideos(data []AwemeData, referer string, cookie string) {
	var (
		i, j, y int
		c chan bool = make(chan bool, 20)
	)
	for y = len(data); i < y; i++ {
		go func(id string, uri string) {
			var e error
			_, e = DownloadAweme(id + ".mp4", uri, referer, cookie)
			if e != nil {
				Mutex.Lock()
				fmt.Println(id, "failed")
				fmt.Fprintln(os.Stderr, e.Error())
				Mutex.Unlock()
			}
			c <- true
		}(data[i].Id, data[i].Uri)
		j++
		if j == cap(c) {
			<-c
			j--
		}
	}
	for j > 0 {
		<-c
		j--
	}
}

func Handle(writer http.ResponseWriter, request *http.Request) {
	var (
		e error
		k bool
		r int
		b []byte
		u0, u1 *url.URL
		t0, t1, t2 string
		f os.FileInfo
		rs *http.Response
		rq *http.Request
		d []AwemeData
	)
	if request.Method == "GET" {
		if len(request.URL.Path) != 1 {
			if request.URL.Path == "/aweme/v1/play/" {
				if t0 = request.Header.Get("X-User-Agent"); len(t0) == 0 {
					t0 = "okhttp"
				}
				t1 = "http://api.tiktokv.com" + request.RequestURI;
				rq, e = http.NewRequest("GET", t1, nil)
				if e != nil {
					goto s400
				}
				rq.Header.Set("User-Agent", t0)
				for ; r < 3; r++ {
					rs, e = http.DefaultTransport.RoundTrip(rq)
					if e != nil {
						goto s400
					} else if rs.StatusCode >= 300 && rs.StatusCode < 400 {
						rs.Body.Close()
						if rs.Header == nil {
							goto s400
						} else if t0 = rs.Header.Get("Location"); len(t0) == 0 {
							goto s400
						} else if u0, e = url.Parse(t0); e != nil {
							goto s400
						} else if u1, e = url.Parse(t1); e != nil {
							goto s400
						}
						writer.Header().Set("Location", u1.ResolveReference(u0).String())
						writer.WriteHeader(302)
						return
					} else if rs.ContentLength > 0 {
						rs.Body.Close()
						writer.Header().Set("Location", t1)
						writer.WriteHeader(302)
						return
					}
					rs.Body.Close()
				}
				goto s400
			}
			goto s404
		}
		f, e = os.Stat("../index.htm")
		if e != nil {
			goto s500
		}
		t0 = "\"" + strconv.FormatInt(f.ModTime().UnixNano() / int64(time.Millisecond), 16) + "\""
		t1 = request.Header.Get("if-none-match")
		if len(t1) > 0 && strings.Index(t1, t0) > -1 {
			writer.Header().Set("Cache-Control", "public, max-age=3600")
			writer.Header().Set("ETag", t0)
			writer.Header().Set("Access-Control-Allow-Origin", "*")
			writer.WriteHeader(http.StatusNotModified)
			return
		}
		writer.Header().Set("Content-Type", "text/html; charset=utf-8")
		writer.Header().Set("Cache-Control", "public, max-age=3600")
		writer.Header().Set("ETag", t0)
		writer.Header().Set("Access-Control-Allow-Origin", "*")
		http.ServeFile(writer, request, "../index.htm")
		return
	} else if request.Method == "POST" {
		if len(request.URL.Path) != 1 {
			goto s404
		}
		request.Body = http.MaxBytesReader(writer, request.Body, 2048)
		b, e = ioutil.ReadAll(request.Body)
		if e != nil {
			goto s400
		}
		u0, e = url.ParseRequestURI(string(b))
		if e != nil {
			goto s400
		} else if _, k = Hosts[u0.Host]; k {
			rq, e = http.NewRequest("GET", string(b), nil)
			if e != nil {
				goto s400
			}
			if t0 = request.Header.Get("X-User-Agent"); len(t0) > 0 {
				rq.Header.Set("User-Agent", t0)
			} else {
				rq.Header.Set("User-Agent", "okhttp")
			}
			if t0 = request.Header.Get("X-Referer"); len(t0) > 0 {
				rq.Header.Set("Referer", t0)
			}
			if t0 = request.Header.Get("X-Cookie"); len(t0) > 0 {
				rq.Header.Set("Cookie", t0)
			}
		} else {
			goto s400
		}
		t0 = request.Header.Get("X-Forwarded-For")
		t1, _, _ = net.SplitHostPort(request.RemoteAddr)
		t2, _, _ = net.SplitHostPort(request.Context().Value(http.LocalAddrContextKey).(net.Addr).String())
		if len(t0) > 0 {
			if len(t1) > 0 && len(t2) > 0 && t1 != t2 {
				rq.Header.Set("X-Forwarded-For", t0 + ", " + t1 + ", " + t2)
			} else {
				rq.Header.Set("X-Forwarded-For", t0 + ", " + t1)
			}
		} else if len(t1) > 0 && len(t2) > 0 && t1 != t2 {
			rq.Header.Set("X-Forwarded-For", t1 + ", " + t2)
		}
		t0 = request.Header.Get("Accept-Encoding")
		if len(t0) > 0 && strings.Index(t0, "gzip") > -1 {
			t0 = "gzip"
			rq.Header.Set("Accept-Encoding", t0)
		} else {
			t0 = ""
		}
		rs, e = http.DefaultClient.Do(rq)
		if e != nil {
			goto s400
		}
		defer rs.Body.Close()
		if rs.StatusCode != http.StatusOK {
			writer.Header().Set("Content-Type", "text/plain; charset=utf-8")
			writer.Write([]byte(rs.Status))
			return
		} else if len(t0) > 0 {
			if t0 = rs.Header.Get("Content-Encoding"); len(t0) > 0 {
				writer.Header().Set("Content-Encoding", t0)
			}
		}
		if t0 = rs.Header.Get("Content-Type"); len(t0) > 0 {
			writer.Header().Set("Content-Type", t0)
		}
		if rs.ContentLength > -1 {
			writer.Header().Set("Content-Length", strconv.FormatInt(rs.ContentLength, 10))
		}
		_, _ = io.Copy(writer, rs.Body)
		return
	} else if request.Method == "PUT" {
		if len(request.URL.Path) != 1 {
			goto s404
		}
		request.Body = http.MaxBytesReader(writer, request.Body, 10000000)
		b, e = ioutil.ReadAll(request.Body)
		if e != nil {
			goto s400
		}
		e = json.Unmarshal(b, &d)
		if e != nil {
			goto s400
		}
		writer.Header().Set("Content-Type", "text/plain; charset=utf-8")
		writer.Write([]byte(http.StatusText(http.StatusOK)))
		go DownloadVideos(d, request.Header.Get("X-Referer"), request.Header.Get("X-Cookie"))
		return
	} else {
		goto s405
	}
	s400:
		writer.Header().Set("Content-Type", "text/plain; charset=utf-8")
		writer.WriteHeader(http.StatusBadRequest)
		writer.Write([]byte(http.StatusText(http.StatusBadRequest)))
		return
	s404:
		writer.Header().Set("Content-Type", "text/plain; charset=utf-8")
		writer.WriteHeader(http.StatusNotFound)
		writer.Write([]byte(http.StatusText(http.StatusNotFound)))
		return
	s405:
		writer.Header().Set("Content-Type", "text/plain; charset=utf-8")
		writer.WriteHeader(http.StatusMethodNotAllowed)
		writer.Write([]byte(http.StatusText(http.StatusMethodNotAllowed)))
		return
	s500:
		writer.Header().Set("Content-Type", "text/plain; charset=utf-8")
		writer.WriteHeader(http.StatusInternalServerError)
		writer.Write([]byte(http.StatusText(http.StatusInternalServerError)))
}

func main() {
	var (
		d string
		e error
		m *http.ServeMux = http.NewServeMux()
		s *http.Server = &http.Server{Handler: m, ReadTimeout: 30 * time.Second}
		l net.Listener;
	)
	d = filepath.Dir(os.Args[0]) + string(os.PathSeparator) + "download";
	e = EnsureDirectory(d)
	if e != nil {
		fmt.Fprintln(os.Stderr, e.Error())
	}
	e = os.Chdir(d)
	if e != nil {
		fmt.Fprintln(os.Stderr, e.Error())
		return
	}
	m.Handle("/", http.HandlerFunc(Handle))
	switch (len(os.Args)) {
		case 2:
			if len(os.Args[1]) == 0 {
				s.Addr = ":http"
			} else {
				s.Addr = os.Args[1]
			}
		default:
			s.Addr = ":http"
	}
	/*e = s.ListenAndServe()
	if e != nil {
		fmt.Fprintln(os.Stderr, e.Error())
	}*/
	l, e = net.Listen("tcp", s.Addr)
	if e != nil {
		fmt.Fprintln(os.Stderr, e.Error())
		return
	}
	fmt.Println(l.Addr().String())
	e = s.Serve(l)
	if e != nil {
		fmt.Fprintln(os.Stderr, e.Error())
	}
}